"use strict";

var nmap = require("nmap");
var AudioDataUtils = require("./AudioDataUtils");

/**
 * @param {function}    decodeFn
 * @param {ArrayBuffer} audioData
 * @param {object}      opts
 * @return {Promise<AudioData>}
 */
function decode(decodeFn, audioData) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  return new Promise(function (resolve, reject) {
    return decodeFn(audioData, opts).then(function (result) {
      if (AudioDataUtils.isAudioData(result)) {
        if (typeof opts.sampleRate === "number") {
          result = resample(result, opts.sampleRate);
        }
        return resolve(result);
      }
      return reject(new TypeError("Failed to decode audio data"));
    }, reject);
  });
}

/**
 * @param {AudioData} audioData
 * @param {number}    sampleRate
 */
function resample(audioData, sampleRate) {
  if (audioData.sampleRate === sampleRate) {
    return audioData;
  }

  var rate = audioData.sampleRate / sampleRate;
  var numberOfChannels = audioData.channelData.length;
  var length = Math.round(audioData.channelData[0].length / rate);
  var channelData = nmap(numberOfChannels, function () {
    return new Float32Array(length);
  });

  for (var i = 0; i < length; i++) {
    var ix = i * rate;
    var i0 = ix | 0;
    var i1 = i0 + 1;
    var ia = ix % 1;

    for (var ch = 0; ch < numberOfChannels; ch++) {
      var v0 = audioData.channelData[ch][i0];
      var v1 = audioData.channelData[ch][i1];

      channelData[ch][i] = v0 + ia * (v1 - v0);
    }
  }

  return { numberOfChannels: numberOfChannels, length: length, sampleRate: sampleRate, channelData: channelData };
}

module.exports = { decode: decode, resample: resample };